//************************************************************************
//*                           BipNUM Win V0.9.2                          *
//*----------------------------------------------------------------------*
//*                      Header file for BipNUM Win                      *
//*                                                                      *
//*                                                                      *
//* Licence : NFM                                                        *
//*                                                         P. Basset    *
//************************************************************************


// include BipNUM only one time
#ifndef _BipNUM_
#define _BipNUM_

#include <iostream>

typedef __int64 LONGLONG;


// global define
const long BP_DECIMAL = 1000000000;		// add to decimal part of currency
const long BP_DECIMALX2 = 2000000000;	// BP_DECIMAL X 2
const int BP_DEC_DIGIT = 9;				// number of digit in decimal part
const LONGLONG BP_MAX_INT = 1000000000000000000;	// max value for integer part
const unsigned long int BP_MAX_DEC = 1999999999;	// max value for decimal part


// structure define
struct BP_STRUCT_CURRENCY			// Currency structure
{
	LONGLONG lBP_Integer;			// integer value
	unsigned long int lBP_Decimal;	// decimal value
	bool bBP_Negative;				// true : the currency is negative
	bool bBP_Null;					// true : the currency is null
};

struct BP_STRUCT_FORMAT				// Format structure
{
	int iBP_NumInt;					// format:number of character before decimal sign
	int iBP_NumDec;					// format:number of character after decimal sign
	std::string sBP_DecimalSign;	// format:decimal sign
	std::string sBP_KiloSign;		// format:kilo sign
	std::string sBP_CurrencySymbol;	// format:currency symbol
	std::string sBP_LeadChar;		// format:leading character (only 1 char)
	std::string sBP_Null;			// null format
	bool bBP_TrailZero;				// trailing zero
};



class BP_Currency
{
	private:
		BP_STRUCT_CURRENCY BP_Cur;	// Currency structure
		BP_STRUCT_FORMAT BP_Fmt;	// Format structure
		int iBPError;				// error code

	public:
		BP_Currency::BP_Currency (); // constructor
		BP_Currency::~BP_Currency (); // destructor
		BP_Currency::set (const std::string BP_SetString);
		BP_Currency::set (const long BP_Long);
		BP_Currency::setnull (void);
		std::string BP_Currency::str (void);
		BP_Currency::setstdformat (const int iBP_SetNumInt,
					  const int iBP_SetNumDec,
					  const std::string sBP_SetFormat);
		BP_Currency::setformat (const int iBP_SetNumInt,
						const int iBP_SetNumDec,
						const std::string sBP_SetDecimalSign,
						const std::string sBP_SetKiloSign,
						const std::string sBP_SetCurrencySymbol,
						const std::string sBP_SetLeadChar,
						const std::string sBP_SetNull,
						const bool bBP_TrailZero);
		BP_Currency::setformat (const std::string sBP_SetFormat);
		BP_Currency::reset (void);
		BP_STRUCT_FORMAT BP_Currency::getformat (void);
		bool BP_Currency::isnull (void);
		BP_Currency::copyvalue (BP_Currency BP_CurSource);
		BP_Currency::copyformat (BP_Currency BP_CurSource);
		int BP_Currency::geterror (void);


		// overload operator : currency
		BP_Currency BP_Currency::operator + (const BP_Currency& BPright);
		BP_Currency BP_Currency::operator - (const BP_Currency& BPright);
		BP_Currency BP_Currency::operator += (const BP_Currency& BPright);
		BP_Currency BP_Currency::operator -= (const BP_Currency& BPright);
		bool BP_Currency::operator == (const BP_Currency& BPright);
		bool BP_Currency::operator != (const BP_Currency& BPright);
		bool BP_Currency::operator > (const BP_Currency& BPright);
		bool BP_Currency::operator >= (const BP_Currency& BPright);
		bool BP_Currency::operator < (const BP_Currency& BPright);
		bool BP_Currency::operator <= (const BP_Currency& BPright);
		BP_Currency BP_Currency::operator ++ (int BPright);
		BP_Currency BP_Currency::operator -- (int BPright);
		friend std::ostream & operator << (std::ostream & os, const BP_Currency& BPright);

		// overload operator : long
		friend BP_Currency BP_Currency::operator + (const BP_Currency& BPleft, const long& BPright);
		friend BP_Currency BP_Currency::operator + (const long& BPleft, const BP_Currency& BPright);
		friend BP_Currency BP_Currency::operator - (const BP_Currency& BPleft, const long& BPright);
		friend BP_Currency BP_Currency::operator - (const long& BPleft, const BP_Currency& BPright);
		bool BP_Currency::operator == (const long& BPright);
		bool BP_Currency::operator != (const long& BPright);
		bool BP_Currency::operator >  (const long& BPright);
		bool BP_Currency::operator >= (const long& BPright);
		bool BP_Currency::operator <  (const long& BPright);
		bool BP_Currency::operator <= (const long& BPright);
};

#endif /* _BipNUM_ */